! lapack_solve resout le systeme ax=b, en dimension n
! - in
! * a et b : matrices du systeme
! * n : dimension du systeme
! - out
! * x : approximation de la solution du systeme via la routine dgesvx 
! * err_min et err_max : le min et le max de l'erreur relative er,
!   avec er=abs((b-ax)/b)
! * f_err et b_err : les erreurs 'forward' et 'backward' de la methode
!
subroutine lapack_solve(a,x,b,n,err_min,err_max,f_err,b_err)
  implicit none
  integer,intent(in) :: n
  real(kind(0.d0)),dimension(n,n),intent(in) :: a
  real(kind(0.d0)),dimension(n),intent(out) :: x
  real(kind(0.d0)),dimension(n),intent(in) :: b
  real(kind(0.d0)),intent(out) :: err_min,err_max,f_err,b_err

  real(kind(0.d0)),dimension(n,n) :: lu
  real(kind(0.d0)),dimension(4*n) :: work
  real(kind(0.d0)),dimension(n) :: r,c
  real(kind(0.d0)),dimension(1) :: ferr,berr
  integer,dimension(n) :: ipiv,iwork
  real(kind(0.d0)) :: rcond
  integer :: nrhs,info
  character :: equed,fact,trans

  real(kind(0.d0)),dimension(n,n) :: a_copy
  real(kind(0.d0)),dimension(n) :: b_copy
  real(kind(0.d0)),dimension(n) :: ax,x_err
  integer :: i

  interface 
     subroutine dgesvx(fact,trans,n,nrhs,a,lda,af,ldaf,ipiv,equed,r,c,b,ldb,x,ldx,rcond,ferr,berr,work,iwork,info)
       character,intent(in) :: fact,trans
       integer,intent(in) :: n,nrhs,lda,ldaf,ldb,ldx
       real(kind(0.d0)),dimension(lda,n),intent(inout) :: a
       real(kind(0.d0)),dimension(ldaf,n),intent(inout) :: af
       integer,dimension(n),intent(inout) :: ipiv
       character,intent(inout) :: equed
       real(kind(0.d0)),dimension(n),intent(inout) :: r,c
       real(kind(0.d0)),dimension(ldb,nrhs),intent(inout) :: b
       real(kind(0.d0)),dimension(ldx,nrhs),intent(out) :: x
       real(kind(0.d0)),intent(out) :: rcond
       real(kind(0.d0)),dimension(nrhs),intent(out) :: ferr,berr
       real(kind(0.d0)),dimension(4*n),intent(out) :: work
       integer,dimension(n),intent(out) :: iwork
       integer,intent(out) :: info
     endsubroutine dgesvx
  endinterface

  ! dgesvx modifie les valeurs de a et b passees en parametres
  ! on fait donc des copies afin de calculer ensuite l'erreur relative
  a_copy=a
  b_copy=b

  fact='N'
  trans='N'
  nrhs=1

  ! appel de la resolution par factorisation LU
  call dgesvx(fact=fact,trans=trans,n=n,nrhs=nrhs,a=a_copy,lda=n,af=lu,ldaf=n,ipiv=ipiv,equed=equed,r=r,c=c,b=b_copy,ldb=n,x=x,ldx=n,rcond=rcond,ferr=ferr,berr=berr,work=work,iwork=iwork,info=info)

  ! calcul de l'erreur relative
  ax=matmul(a,x)
  do i=1,n
     x_err(i)=abs((b(i)-ax(i))/b(i))
  enddo

  err_min=minval(x_err)
  err_max=maxval(x_err)
  f_err=ferr(1)
  b_err=berr(1)

  return
endsubroutine lapack_solve
