! compare renvoie les resultats de la resolution de AX=B par deux methodes
! les parametres d'appel, recuperes dans un fichier texte, sont :
! * la dimension du systeme
! * '-data' si l'on ne souhaite que les donnees pertinentes,
!   ie les bornes d'erreur
!
program compare
  use misc_interface
  use lapack_interface
  use st_interface
  use cadna
  implicit none
  real(kind(0.d0)),allocatable,dimension(:,:) :: a
  real(kind(0.d0)),allocatable,dimension(:) :: b,x
  type(double_st),allocatable,dimension(:,:) :: a_st
  type(double_st),allocatable,dimension(:) :: b_st,x_st
  integer :: i,j,n,m,as,deci_max,deci_min
  real(kind(0.d0)) :: err_min_l,err_max_l,f_err,b_err
  real(kind(0.d0)) :: err_min_g,err_max_g
  logical :: data_flag

  call cadna_init(-1)

  ! recuperation des parametres de ligne de commande
  n=i_get_param('compare.params',1)
  i=get_params_nb('compare.params')
  if(i<2) then
     data_flag=.false.
  else
     data_flag=(c_get_param('compare.params',2).eq.'-data')
  endif

  if(n.eq.0) then
     ! en-tete des fichiers de donnees
     if(data_flag) write(unit=*,fmt='(a)') '#N ERR_MIN_L              ERR_MAX_L              F_ERR_L                B_ERR_L                ERR_MIN_G              ERR_MAX_G                DMIN DMAX'

  elseif(n.ge.1) then

     ! l'allocation n'est pas controlée, pour alléger le code
     allocate(a(n,n))
     allocate(a_st(n,n))
     allocate(b(n))
     allocate(b_st(n))
     allocate(x(n))
     allocate(x_st(n))

     ! affectation de A et B
     do i=1,n
        do j=1,n
           a(i,j)=1./(i+j-1)
           a_st(i,j)=1./(i+j-1)
        enddo
     enddo
     m=floor(real(n/2))
     do i=m+1,n
        do j=m+1,n
           a(i,j)=a(i,j)+1./(i-m+j-m-1)
           a_st(i,j)=a_st(i,j)+1./(i+j-1)
        enddo
     enddo
     do i=1,n
        b(i)=i
        b_st(i)=i
     enddo

     ! resolution du systeme
     call lapack_solve(a,x,b,n,err_min_l,err_max_l,f_err,b_err)
     call st_solve(a_st,x_st,b_st,n,err_min_g,err_max_g,deci_min,deci_max)

     if(data_flag) then
        ! renvoi des erreurs
        write(unit=*,fmt='(i2,6e23.16,2i5)') n,err_min_l,err_max_l,f_err,b_err,err_min_g,err_max_g,deci_min,deci_max
     else
        ! renvoi du resultat formate
        write(unit=*,fmt='(i2,a)') n,'=============================================================='
        write(unit=*,fmt='(a,e25.16,e25.16)') '  LAPACK ERR |',err_min_l,err_max_l
        write(unit=*,fmt='(a,e25.16,e25.16)') 'LPCK FWD/BWD |',f_err,b_err
        write(unit=*,fmt='(a)') '----------------------------------------------------------------'
        write(unit=*,fmt='(a,e25.16,e25.16)') 'GAUSS_ST ERR |',err_min_g,err_max_g
        write(unit=*,fmt='(a,i25,i25)') 'DECI MIN/MAX |',deci_min,deci_max
        write(unit=*,fmt='(a)') '---------------DGESVX--------------------GAUSS_ST---------------'
        do i=1,n
           write(unit=*,fmt='(i12,e25.16,a,a)') i,x(i),'   ',str(x_st(i))
        enddo
     endif
  endif

endprogram compare
