! st_solve resout le systeme ax=b, en dimension n
! - in
! * a et b : matrices du systeme
! * n : dimension du systeme
! - out
! * x : approximation de la solution du systeme via la routine dgesvx 
! * err_min et err_max : le min et le max de l'erreur relative er,
!   avec er=abs((b-ax)/b)
! * deci_min et deci_max : le min et max du nombre de chiffres exacts
!
subroutine st_solve(a,x,b,n,err_min,err_max,deci_min,deci_max)
  use cadna
  implicit none
  integer,intent(in) :: n
  type(double_st),dimension(n,n),intent(in) :: a
  type(double_st),dimension(n),intent(out) :: x
  type(double_st),dimension(n),intent(in) :: b
  real(kind(0.d0)),intent(out) :: err_min,err_max
  integer,intent(out) :: deci_min,deci_max

  type(double_st) :: det

  real(kind(0.d0)),dimension(n,n) :: a2
  real(kind(0.d0)),dimension(n) :: ax,x_err,b2,x2
  integer,dimension(n) :: deci
  type(double_st),dimension(n,n) :: a_copy
  type(double_st),dimension(n) :: b_copy
  integer :: i,j

  interface 
     subroutine gauss_st(a,b,n,det)
       use cadna
       integer,intent(in) :: n
       type(double_st),dimension(n,n),intent(inout) :: a
       type(double_st),dimension(n),intent(inout) :: b
       type(double_st),intent(out) :: det
     endsubroutine gauss_st
  endinterface

  ! gauss_st modifie les valeurs de a et b passees en parametres
  ! on fait donc des copies afin de calculer ensuite l'erreur relative
  a_copy=a
  b_copy=b

  ! appel de la resolution par Gauss
  call gauss_st(a_copy,b_copy,n,det)
  x=b_copy

  ! conversion en flottants et recuperation des chiffres significatifs
  do i=1,n
     do j=1,n
        a2(i,j)=a(i,j)
     enddo
     x2(i)=x(i)
     b2(i)=b(i)
     deci(i)=cestac(x(i))
  enddo

  ! calcul de l'erreur relative
  ax=matmul(a2,x2)
  do i=1,n
     x_err(i)=abs((b2(i)-ax(i))/b2(i))
  enddo

  err_min=minval(x_err)
  err_max=maxval(x_err)
  deci_min=minval(deci)
  deci_max=maxval(deci)

  return
endsubroutine st_solve
